#
# (c) 2022 Giorgio Gonnella, University of Goettingen, Germany
#
import pytest
from pathlib import Path

@pytest.fixture
def a_file_in():
  def _a_file_in(dirname, notin=[]):
    filenames = [f.name for f in Path(dirname).glob("*") if f.is_file()]
    for dirname in notin:
      for fn in filenames:
        if Path(dirname).joinpath(fn).exists():
          filenames.remove(fn)
    if filenames:
      return filenames[0]
    else:
      return None
  return _a_file_in



import sys
import xdg
import inspect
from pathlib import Path

def findconfig(filename, allow_dot=True, use_xdg=True, use_home=True,
                         use_src=True, more_paths = [], src_climb = 1,
                         exception = False, verbose = False):
  """
  Find a config file.

  :param filename: The filename to search for.
  :param allow_dot: Whether to allow a prepended dot to the filename.
  :param use_xdg: Whether to search in the XDG config directories.
  :param use_home: Whether to search in the user home directory.
  :param use_src: Whether to search in the source directory of the caller module
                  and in its ancestor directories, until they contain a
                  __init__.py file, plus additional src_levels levels.
  :param src_climb: The number of levels to go up in the caller module's source
                    directory, when there is no __init__.py (default: 1).
  :param more_paths: A list of additional paths to search in (default: [])
  :param exception: Whether to raise an exception if the file is not found
                    (default: False).
  :param verbose: Whether to print the found path to stderr (default: False).
  :return: The path to the file, or None if not found.
  """
  search_paths = []
  assert(filename)
  if allow_dot:
    if filename[0] == ".":
      allow_dot = False
  if use_xdg:
    search_paths.append(xdg.xdg_config_home())
    search_paths += xdg.xdg_config_dirs()
  if use_home:
    search_paths.append(Path.home())
  if use_src:
    stk = inspect.stack()
    if len(stk) > 1:
      srcdir = Path(stk[1].filename).parent
      while srcdir.joinpath('__init__.py').exists():
        search_paths.append(srcdir)
        srcdir = srcdir.parent
      for i in range(src_climb):
        search_paths.append(srcdir)
        srcdir = srcdir.parent
      search_paths.append(srcdir)
  search_paths += [Path(p) for p in more_paths]
  for path in search_paths:
    if path.is_dir():
      filenames = [path / filename]
      if allow_dot:
        filenames.append(path / f".{filename}")
      for f in filenames:
        if f.is_file():
          if verbose:
            print(f"Configuration file: {f}", file=sys.stderr)
          return f
  if exception:
    if allow_dot:
      filename = f"{filename} or .{filename}"
    search_paths_formatted = '\n'.join([f"  {p.resolve()}" for p in search_paths])
    raise FileNotFoundError(f"No file {filename} could be found "+\
                            f"in any of the following paths:\n"+\
                            search_paths_formatted)
  return None



#
# (c) 2022 Giorgio Gonnella, University of Goettingen, Germany
#

import pytest
from findconfig import findconfig
from pathlib import Path
from xdg import xdg_config_home, xdg_config_dirs
import warnings

def test_findconfig_xdg_config_home(a_file_in):
  filename = a_file_in(xdg_config_home())
  if filename is None:
    warnings.warn("No file is XDG config home directory found, test skipped")
    return
  assert findconfig(filename) == Path(xdg_config_home()) / filename

def test_findconfig_xdg_config_dirs(a_file_in):
  xcd = xdg_config_dirs()
  if not xcd:
    warnings.warn("No XDG config dirs found, test skipped")
    return
  filename = a_file_in(xcd[0], notin=[xdg_config_home()])
  if filename is None:
    warnings.warn("No file is XDG config dirs found, test skipped")
    return
  assert findconfig(filename) == Path(xcd[0]) / filename

def test_findconfig_home(a_file_in):
  filename = a_file_in(Path.home(), notin=[xdg_config_home()]+xdg_config_dirs())
  if filename is None:
    warnings.warn("No file is home directory found, test skipped")
    return
  assert findconfig(filename) == Path.home() / filename

def test_findconfig_src(a_file_in):
  import findconfig_caller.call
  assert findconfig_caller.call.find(0) == \
      Path(__file__).parent / "findconfig_testfile_0"
  assert findconfig_caller.call.find(1) == \
      Path(__file__).parent / "findconfig_caller" / "findconfig_testfile_1"
  assert findconfig_caller.call.find(2) == None
  import findconfig_caller.level2.call
  assert findconfig_caller.level2.call.find(0) == \
      Path(__file__).parent / "findconfig_testfile_0"
  assert findconfig_caller.level2.call.find(1) == \
      Path(__file__).parent / "findconfig_caller" / "findconfig_testfile_1"
  assert findconfig_caller.level2.call.find(2) == \
      Path(__file__).parent / "findconfig_caller" / "level2" / \
        "findconfig_testfile_2"

def test_findconfig_more_paths():
  custompath = Path(__file__).parent / "custom"
  findconfig("find", more_paths=[custompath]) == custompath / \
    "findconfig_testfile_C"

def test_findconfig_src_climb():
  import custompath.level2.call
  assert custompath.level2.call.find(0, 2) == \
      Path(__file__).parent / "findconfig_testfile_0"
  assert custompath.level2.call.find(0, 1) == None
  assert custompath.level2.call.find("C", 1) == \
      Path(__file__).parent / "custompath" / "findconfig_testfile_C"
  assert custompath.level2.call.find("C", 0) == None
  import custompath.level2.level3.call
  assert custompath.level2.level3.call.find("C", 1) == \
      Path(__file__).parent / "custompath" / "findconfig_testfile_C"
  assert custompath.level2.level3.call.find("C", 0) == None
  assert custompath.level2.level3.call.find(0, 2) == \
      Path(__file__).parent / "findconfig_testfile_0"
  assert custompath.level2.level3.call.find(0, 1) == None

def test_findconfig_allowdot():
  assert findconfig("findconfig_testfile_hidden") == \
      Path(__file__).parent / ".findconfig_testfile_hidden"
  assert findconfig("findconfig_testfile_hidden", allow_dot = False) == None

def test_exception():
  assert findconfig("findconfig_testfile_not_existing") == None
  with pytest.raises(FileNotFoundError):
    findconfig("findconfig_testfile_not_existing", exception = True)
